"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateFilesFromTemplate = exports.generateDataBind = exports.isSlot = exports.runGenerateCore = void 0;
const weapps_core_1 = require("../../weapps-core");
const common_1 = require("../types/common");
const path_1 = __importDefault(require("path"));
const template_1 = __importDefault(require("../template"));
const util_1 = require("../util");
const config_1 = require("../config");
const style_1 = require("../util/style");
const types_1 = require("../../types");
const dataSource_1 = require("../../utils/dataSource");
const cals_1 = require("@cloudbase/cals");
const lodash_1 = require("lodash");
const _1 = require(".");
const common_2 = require("../util/common");
function deserializeCodeResource(_, data) {
    const { type, codeType, ...resData } = data;
    return {
        ...resData,
        type: codeType,
    };
}
async function runGenerateCore(props) {
    const { appBuildDir, cals, sub_cals = [], appData, dependencies = [], buildTypeList, appKey, basename, fileCodeMap, isSandbox = false, deployOptions, _indexPage, processCssUnit, rpxCalcRoot, isPrivateMode = false, expirationStartTimesnap, } = props;
    const calses = [cals].concat(sub_cals || []);
    // 安装插件依赖
    const deps = calses.reduce((deps, app) => {
        var _a;
        Object.entries(((_a = app.extra) === null || _a === void 0 ? void 0 : _a.npmDependencies) || {}).forEach(([name, version]) => {
            deps[name] = version;
        });
        return deps;
    }, {});
    // 合并组件库的公共npm
    dependencies.map((compLb) => {
        if (compLb.isComposite && compLb.compLibCommonResource) {
            Object.assign(deps, compLb.compLibCommonResource.npm);
        }
    });
    /**
     * 组模块目录结构待定
     * 不生成子包
     */
    const usedComps = props.usedComps
        ? props.usedComps
        : (0, cals_1.getUsedComps)(calses, dependencies, {
            /**
             * withAction 必须为 true
             * 此时才会启用新的返回格式
             */
            withAction: true,
            streamline: true,
        });
    const { rootPath = '' } = appData;
    console.log(`Generating files for ${rootPath ? `Sub app ${rootPath}` : 'Main app'}`);
    const dstDir = path_1.default.join(appBuildDir, 'src', rootPath ? `${rootPath}` : '');
    await generateCodeFromTpl(appKey, calses, dstDir, fileCodeMap, isSandbox, deployOptions, dependencies, processCssUnit, isPrivateMode, usedComps);
    await writeLowCodeFiles(calses, dstDir, fileCodeMap, processCssUnit, rpxCalcRoot);
    await generateAppData(cals, sub_cals, dependencies, appBuildDir, deployOptions, usedComps, fileCodeMap);
    await generateRouterFile(appBuildDir, basename, buildTypeList, fileCodeMap, isSandbox, _indexPage, expirationStartTimesnap);
}
exports.runGenerateCore = runGenerateCore;
/**
 * 根据复合组件库获取到所有包含抽象节点的数据
 * @param dependencies
 * @returns
 */
function getGenericCompFromDep(dependencies = []) {
    const genericCompMap = {};
    dependencies.forEach((compLib) => {
        if (compLib.isComposite) {
            compLib.components.forEach((component) => {
                (0, util_1.deepDealComponentSchemaJson)(Object.values(component.componentInstances)[0], (componentSchema) => {
                    if (componentSchema === null || componentSchema === void 0 ? void 0 : componentSchema.genericComp) {
                        genericCompMap[`${component.materialName}:${component.name}`] = componentSchema.genericComp;
                    }
                });
            });
        }
    });
    return genericCompMap;
}
function pullComponentToListByInstance(sourceKey, originComponentList, fixedDependencies) {
    var _a;
    const { materialName, name, variableName } = (0, util_1.getMetaInfoBySourceKey)(sourceKey);
    const componentKey = `${materialName}_${name}`;
    const isExistComponent = originComponentList.find((item) => item.key === componentKey);
    if (!isExistComponent) {
        const foundOne = fixedDependencies.find((m) => m.name === materialName);
        if (!foundOne)
            return;
        const { schemaVersion = '' } = foundOne;
        let isPlainProps = false;
        try {
            if (Number((_a = schemaVersion.split('.')) === null || _a === void 0 ? void 0 : _a[0]) >= 3) {
                isPlainProps = true;
            }
        }
        catch (e) { }
        originComponentList.push({
            sourceKey,
            name: name || '',
            materialName: materialName || '',
            materialVersion: foundOne.version,
            isComposite: foundOne.isComposite,
            key: componentKey,
            variableName: variableName || '',
            isPlainProps,
            entries: foundOne === null || foundOne === void 0 ? void 0 : foundOne.entries,
        });
    }
}
function isSlot(comp) {
    return comp.path && !comp['x-props'];
}
exports.isSlot = isSlot;
function generateDataBind(bind) {
    let expr = '';
    const { type, value } = bind;
    if (type === weapps_core_1.PropBindType.forItem) {
        expr = `forItems.${value}`;
    }
    else if (type === weapps_core_1.PropBindType.expression) {
        expr = value;
    }
    else if (type === weapps_core_1.PropBindType.prop) {
        const isNegated = value.startsWith('!');
        const originValue = value.replace(/^!/, '');
        expr = `${isNegated ? '!' : ''}$comp.props.data.${originValue}`;
    }
    else {
        const isGlobal = value.startsWith('global.');
        const isCopositive = value.startsWith('comp-') || value.startsWith('$comp_');
        const PREFIX_MAP = {
            [weapps_core_1.PropBindType.state]: 'state',
            [weapps_core_1.PropBindType.computed]: 'computed',
            [weapps_core_1.PropBindType.dataVar]: 'dataVar',
            [weapps_core_1.PropBindType.stateData]: 'dataset.state',
            [weapps_core_1.PropBindType.paramData]: 'dataset.params',
        };
        switch (type) {
            case weapps_core_1.PropBindType.state:
            case weapps_core_1.PropBindType.computed:
            case weapps_core_1.PropBindType.dataVar:
            case weapps_core_1.PropBindType.stateData:
            case weapps_core_1.PropBindType.paramData: {
                if (isCopositive) {
                    expr = value
                        .replace(/^comp-\w+/, `$comp.${PREFIX_MAP[type]}`)
                        .replace(/^\$comp_\w+/, `$comp.${PREFIX_MAP[type]}`);
                }
                else {
                    expr = value
                        .replace(/^\w+./, `${isGlobal ? 'app' : '$page'}.${PREFIX_MAP[type]}.`)
                        .replace(/^comp-\w+/, `$comp.${PREFIX_MAP[type]}`)
                        .replace(/^\$comp_\w+/, `$comp.${PREFIX_MAP[type]}`);
                }
            }
        }
    }
    return expr;
}
exports.generateDataBind = generateDataBind;
async function generateRouterFile(appBuildDir, basename = '', buildTypeList = [], fileCodeMap, isSandbox, _indexPage = false, expirationStartTimesnap = 0) {
    fileCodeMap = generateFilesFromTemplate({
        templatesData: {
            'router/index.tpl.jsx': {
                expirationStartTimesnap,
                basename,
                isHash: isSandbox,
                context: {
                    history: { _indexPage },
                    basic: (0, common_1.buildAsXPageByBuildType)(buildTypeList),
                    checkPageAuth: (0, common_1.buildAsXPageByBuildType)(buildTypeList) || (0, common_1.buildAsAdminPortalByBuildType)(buildTypeList)
                        ? 'function(){ return true }'
                        : 'undefined',
                },
            },
        },
        fileCodeMap,
        baseTargetPath: path_1.default.resolve(appBuildDir, 'src'),
    });
}
async function generateAppData(cals, sub_cals = [], dependencies, appBuildDir, deployOptions, usedComps, fileCodeMap) {
    function streamlineCals(cals) {
        return {
            ...cals,
            resources: undefined,
            items: (cals.items || []).map((page) => {
                return {
                    ...page,
                    resources: undefined,
                };
            }),
            extra: {
                ...(cals.extra || {}),
                // 过滤设计态属性
                pageTree: undefined,
                compHiddenConfig: undefined,
                componentPropsDatas: undefined,
                processPages: undefined,
                lastDependencies: undefined,
                conditionalRendering: undefined,
                dataForm: undefined,
                compConfig: undefined,
            },
        };
    }
    fileCodeMap = generateFilesFromTemplate({
        templatesData: {
            'router/data.tpl.js': {
                /**
                 * 不添加缩进以精简大小
                 */
                cals: JSON.stringify({
                    ...streamlineCals(cals),
                    version: (deployOptions === null || deployOptions === void 0 ? void 0 : deployOptions.version) || '',
                    dataSources: [],
                }),
                sub_cals: JSON.stringify((sub_cals === null || sub_cals === void 0 ? void 0 : sub_cals.map((cals) => streamlineCals(cals))) || []),
                /**
                 * 不添加缩进以精简大小
                 */
                dependencies: JSON.stringify(dependencies
                    .filter((lib) => { var _a, _b; return lib.name === config_1.OFFICIAL_LIB_KEY || !!((_a = usedComps === null || usedComps === void 0 ? void 0 : usedComps.component) === null || _a === void 0 ? void 0 : _a[lib.name]) || !!((_b = usedComps === null || usedComps === void 0 ? void 0 : usedComps.action) === null || _b === void 0 ? void 0 : _b[lib.name]); })
                    .map((lib) => {
                    return {
                        ...lib,
                        components: [],
                        actions: undefined,
                        srcZipUrl: undefined,
                        mpPkgUrl: undefined,
                        mainJsPath: undefined,
                        controlJsPath: undefined,
                        mainCssPath: undefined,
                        entries: undefined,
                        plugins: undefined,
                    };
                })),
            },
        },
        fileCodeMap,
        baseTargetPath: path_1.default.resolve(appBuildDir, 'src'),
    });
}
async function writeLowCodeFiles(calses, appBuildDir, fileCodeMap, processCssUnit, rpxCalcRoot) {
    const lowcodeRootDir = path_1.default.join(appBuildDir, 'lowcode');
    let themeCode;
    /**
     * 写包级别 common lowcode
     * 需要先获取 common 级别的 themeCode
     */
    await Promise.all(calses.map(async (cals) => {
        var _a, _b, _c;
        const { rootPath = '' } = cals.extra || {};
        const outDir = rootPath ? path_1.default.join(appBuildDir, 'packages', rootPath, 'lowcode') : lowcodeRootDir;
        fileCodeMap = await generateFilesFromTemplate({
            templatesData: {
                'app/handlers.tpl.js': {
                    pageModules: (_a = cals.items) === null || _a === void 0 ? void 0 : _a.reduce((map, page) => {
                        var _a, _b;
                        const uuid = rootPath ? `${rootPath}/${page.id}` : page.id;
                        map[page.id] = {
                            pageNameVar: (0, common_2.normalizePageId)(uuid),
                            codes: (((_b = (_a = page.resources) === null || _a === void 0 ? void 0 : _a.filter) === null || _b === void 0 ? void 0 : _b.call(_a, (mod) => mod.name !== cals_1.ECodeName.PLACEHOLDER && mod.codeType === cals_1.ECodeType.HANDLER_FN)) || []).map((item) => deserializeCodeResource(undefined, item)),
                        };
                        return map;
                    }, {}),
                },
            },
            fileCodeMap,
            baseTargetPath: rootPath ? path_1.default.join(appBuildDir, 'packages', rootPath) : appBuildDir,
        });
        let globalImports = '';
        let globalExports = [];
        if (!rootPath) {
            const codeModules = (0, cals_1.processRuntimeCodeResources)({ id: 'global' }, ((_b = cals === null || cals === void 0 ? void 0 : cals.resources) === null || _b === void 0 ? void 0 : _b.filter((item) => item.type === cals_1.EResourceType.CODE && item.name !== cals_1.ECodeName.PLACEHOLDER).map((item) => deserializeCodeResource(undefined, item))) || [], 'app');
            themeCode = codeModules.find((mod) => mod.type === cals_1.ECodeType.THEME);
            await Promise.all(codeModules.map((m) => writeCode2file(outDir, m, 'global', undefined, processCssUnit, rpxCalcRoot)).concat());
            const styleModule = codeModules.find((mode) => mode.type === cals_1.ECodeType.STYLE);
            globalImports = ['state', 'computed', 'lifecycle']
                .map((key) => `import ${key} from './${key}'`)
                .concat("import common from '../app/common'")
                .concat('')
                .join(';\n');
            globalExports = [
                `global: {
state,
computed,
lifecycle,
style: [{code:\`${styleModule
                    ? ((_c = fileCodeMap[path_1.default.join(outDir, (0, weapps_core_1.getCodeModuleFilePath)('global', styleModule, {
                        style: '.css',
                    }))]) === null || _c === void 0 ? void 0 : _c.code) || ''
                    : ''}\`}],
          common
        }`,
            ];
        }
        await writeCode2file(outDir, {
            type: 'index',
            name: 'entry',
            path: 'global/index',
            code: `${globalImports}${(cals.items || [])
                .map((page) => {
                const uuid = rootPath ? `${rootPath}/${page.id}` : page.id;
                return `import * as ${(0, common_2.normalizePageId)(uuid)} from '${`./${page.id}/index`}'`;
            })
                .concat('')
                .join(';\n')}
    export default {
      ${globalExports
                .concat((cals.items || []).map((page) => {
                const uuid = rootPath ? `${rootPath}/${page.id}` : page.id;
                return `['${uuid}']: ${(0, common_2.normalizePageId)(uuid)}`;
            }))
                .join(',\n')}
    }
    `,
        }, 'global', undefined, processCssUnit, rpxCalcRoot);
    }));
    /**
     * 写各个包页面级别lowcode
     */
    await Promise.all(calses.reduce((list, cals) => {
        const { rootPath = '' } = cals.extra || {};
        const outDir = rootPath ? path_1.default.join(appBuildDir, 'packages', rootPath, 'lowcode') : lowcodeRootDir;
        list.push(...(cals.items || []).map(async (page) => {
            var _a, _b;
            const codeModules = (0, cals_1.processRuntimeCodeResources)({ id: page.id }, ((_a = page === null || page === void 0 ? void 0 : page.resources) === null || _a === void 0 ? void 0 : _a.filter((item) => item.type === cals_1.EResourceType.CODE && item.name !== cals_1.ECodeName.PLACEHOLDER).map((item) => deserializeCodeResource(undefined, item))) || [], 'page');
            await Promise.all(codeModules.map((m) => writeCode2file(outDir, m, page.id, page, processCssUnit, rpxCalcRoot)));
            const styleModule = codeModules.find((mode) => mode.type === cals_1.ECodeType.STYLE);
            await writeCode2file(outDir, {
                type: 'index',
                name: 'entry',
                path: `${page.id}/index`,
                code: `export { default as state } from './state';
          export { default as  computed } from './computed';
          export { default as lifecycle } from './lifecycle';
          // export style from './style';
          export const style = [{code: \`${styleModule
                    ? ((_b = fileCodeMap[path_1.default.join(outDir, (0, weapps_core_1.getCodeModuleFilePath)(page.id, styleModule, {
                        style: '.css',
                    }))]) === null || _b === void 0 ? void 0 : _b.code) || ''
                    : ''}\`}];
          export { ${(0, common_2.normalizePageId)(rootPath ? `${rootPath}/${page.id}` : page.id)} as handler} from '../../app/handlers';
          `,
            }, page.id, page, processCssUnit, rpxCalcRoot);
        }));
        return list;
    }, []));
    async function writeCode2file(destDir, mod, pageId, page, processCssUnit = 'rem', rpxCalcRoot) {
        const file = path_1.default.join(destDir, (0, weapps_core_1.getCodeModuleFilePath)(pageId, mod, {
            style: '.css',
        }));
        let code = mod.code || '';
        let weappsApiPrefix = '';
        if (mod.type !== cals_1.ECodeType.THEME && mod.type !== cals_1.ECodeType.STYLE && mod.type !== 'index') {
            const isIncludeAppDot = !!code
                // 先去掉模板代码
                .replace('通过 app.common.[name].xxx 访问这里定义的方法或值', '')
                // 再匹配
                .match(/[^'"\w]?app[^'"\w]/g);
            if (isIncludeAppDot) {
                // weappsApiPrefix = `import { app } from '${path
                //   .relative(path.dirname(file), `${appBuildDir}/app/global-api`)
                //   .replace(/\\/g, '/')}';\nconst $app = app;\n`;
                // weappsApiPrefix = `const app = window.app;\nconst $app = app;\n`;
                weappsApiPrefix = `${
                /* !isSandbox */ false
                    ? `import { app } from '${path_1.default
                        .relative(path_1.default.dirname(file), `${appBuildDir}/app/global-api`)
                        .replace(/\\/g, '/')}';`
                    : config_1.LOWCODE_INJECT_APP_TEMPLATE}\nconst $app = new Proxy({}, { get: function(obj, prop){ return app[prop] }});`;
            }
            if (page) {
                // weappsApiPrefix = `import { $page } from '${(path.dirname(file), appBuildDir).replace(
                //   /\\/g,
                //   '/',
                // )}/pages/${pageId}/api';`;
                // weappsApiPrefix = `cosnt $page = window.$page`;
            }
        }
        if (mod.type === 'style') {
            // 生成页面样式
            code = await (0, style_1.processLess)(((themeCode === null || themeCode === void 0 ? void 0 : themeCode.code) || '') + code, processCssUnit);
        }
        if (mod.type === 'theme') {
            return;
        }
        fileCodeMap[file] = {
            code: `${weappsApiPrefix}\n${code}`,
        };
    }
}
async function generateCodeFromTpl(
/**
 * 应用ID
 */
appKey, 
/**
 * 应用数据
 */
calses, appBuildDir, fileCodeMap, isSandbox, deployOptions, dependencies, processCssUnit, isPrivateMode, usedCmps) {
    var _a, _b, _c;
    const { staticDatasourceProfile = true } = (0, cals_1.parseVersion)('', dependencies);
    const cals = calses[0];
    const originComponentList = [];
    const useMaterialActions = [];
    /**
     * 有 used 获取 originComponentList
     */
    if (usedCmps.component) {
        // eslint-disable-next-line no-restricted-syntax
        for (const libName in usedCmps.component) {
            if (libName !== '$template') {
                for (let item of Array.from(usedCmps.component[libName])) {
                    const sourceKey = `${libName}:${item}`;
                    pullComponentToListByInstance(sourceKey, originComponentList, dependencies);
                }
            }
        }
    }
    if (usedCmps.action) {
        // eslint-disable-next-line no-restricted-syntax
        for (const libName in usedCmps.action) {
            if (libName !== '$template') {
                for (let actionName of Array.from(usedCmps.action[libName])) {
                    const sourceKey = `${libName}:${actionName}`;
                    const material = dependencies.find((m) => m.name === libName);
                    if (!material) {
                        continue;
                    }
                    const { materialName, name, variableName } = (0, util_1.getMetaInfoBySourceKey)(sourceKey);
                    useMaterialActions.push({
                        name,
                        materialName,
                        materialVersion: material === null || material === void 0 ? void 0 : material.version,
                        key: `${materialName}_${name}`,
                        sourceKey,
                        type: weapps_core_1.ActionType.Material,
                        variableName,
                        entries: material === null || material === void 0 ? void 0 : material.entries,
                    });
                }
            }
        }
    }
    const useComponents = [];
    originComponentList.reduce((map, compItem) => {
        const sourceKey = `${compItem.materialName}:${compItem.name}`;
        if (!map[sourceKey]) {
            useComponents.push(compItem);
        }
        return map;
    }, {});
    const originEntryMap = {};
    useComponents.forEach((component) => {
        if (!originEntryMap[component.materialName]) {
            const info = {
                materialName: component.materialName,
                version: component.materialVersion,
                entries: component.entries,
            };
            originEntryMap[component.materialName] = info;
        }
    });
    const officialLib = dependencies.find((lib) => lib.name === config_1.OFFICIAL_LIB_KEY);
    useMaterialActions.forEach((action) => {
        if (!originEntryMap[action.materialName]) {
            const info = {
                materialName: action.materialName,
                version: action.materialVersion,
                entries: action.entries,
            };
            originEntryMap[action.materialName] = info;
        }
    });
    if (officialLib && !originEntryMap[config_1.OFFICIAL_LIB_KEY]) {
        originEntryMap[config_1.OFFICIAL_LIB_KEY] = {
            materialName: officialLib.name,
            version: officialLib.version,
            entries: officialLib.entries,
        };
    }
    const entryImportStringArr = isSandbox ? [] : getEntryImportStringArr(Object.values(originEntryMap));
    const componentImportStringArr = isSandbox
        ? useComponents.map((compItem) => {
            return `const ${(0, util_1.upperFirst)(compItem.variableName)} = window["@weapps-materials-main-${compItem.materialName}"].components["${compItem.name}"]`;
        })
        : getComponentImportStringArr(useComponents);
    const actionImportStringArr = isSandbox
        ? useMaterialActions.map((action) => {
            return `const ${action.variableName} = window["@weapps-materials-main-${action.materialName}"].actions["${action.name}"]`;
        })
        : getActionImportStringArr(useMaterialActions);
    unshiftAllOfficialActionToImportStringArr(officialLib, actionImportStringArr);
    // # all templates to be generated
    const templatesData = {
        'app/common.tpl.js': {
            mods: (_a = cals.resources) === null || _a === void 0 ? void 0 : _a.filter((m) => m.codeType === cals_1.ECodeType.NORMAL_MODULE && m.name !== cals_1.ECodeName.PLACEHOLDER).map((m) => m.name),
        },
        'datasources/config.tpl.js': {
            appID: appKey,
            envID: cals.envId || cals.extra.envId,
            isProd: (deployOptions === null || deployOptions === void 0 ? void 0 : deployOptions.mode) === types_1.DEPLOY_MODE.UPLOAD,
            clientID: ((_b = cals.extra) === null || _b === void 0 ? void 0 : _b.enableLoginStatusShare) ? cals.envId || cals.extra.envId : (_c = cals.extra) === null || _c === void 0 ? void 0 : _c.clientId,
            isPrivateMode,
        },
        'datasources/datasource-profiles.tpl.js': {
            datasourceProfiles: JSON.stringify((0, dataSource_1.getDatasourceProfiles)(
            /**
             * 精简文件生成，新 cloud-sdk 不依赖 profile 了
             * 2022.08.24 设计态获取数据 outParams.propterties 运行态数据 outParams.items
             * 因此继续使用设计态数据
             * 2023.05.26 新版组件不再依赖此类数据，为了精简，判断不再注入
             */
            // [],
            staticDatasourceProfile ? cals.dataSources || cals.datasources || [] : []), null, 2),
        },
        'datasources/dataset-profiles.tpl.js': {
            datasetProfiles: (0, util_1.JsonToStringWithVariableName)((0, dataSource_1.getDatasetProfiles)({
                dataset: calses[0].dataset,
            }, calses.map((cals) => {
                var _a, _b;
                return {
                    dataset: cals.dataset,
                    rootPath: ((_a = cals.extra) === null || _a === void 0 ? void 0 : _a.rootPath) || '',
                    pageInstanceList: ((_b = cals.items) === null || _b === void 0 ? void 0 : _b.map((page) => {
                        return { dataset: page.dataset, id: page.id };
                    })) || [],
                };
            }))),
        },
        'utils/adapter/dependencies.tpl.jsx': {
            upperFirst: util_1.upperFirst,
            OFFICIAL_LIB_KEY: config_1.OFFICIAL_LIB_KEY,
            entryImports: entryImportStringArr.join(';\n'),
            componentImports: componentImportStringArr.join(';\n'),
            actionImports: actionImportStringArr.join(';\n'),
            useComponents,
            useMaterialActions,
            controlMap: JSON.stringify(dependencies.reduce((map, lib) => {
                map[`${lib.name}@${lib.version}`] = {
                    schemaVersion: lib.schemaVersion,
                    components: lib.components
                        .filter((item) => {
                        var _a, _b, _c;
                        return (_c = (_b = (_a = usedCmps === null || usedCmps === void 0 ? void 0 : usedCmps.component) === null || _a === void 0 ? void 0 : _a[lib.name]) === null || _b === void 0 ? void 0 : _b.has) === null || _c === void 0 ? void 0 : _c.call(_b, item.name);
                    })
                        .map((item) => {
                        const { name, dataForm, properties, emitEvents, events, meta, compConfig, inputProps, syncProps, isContainer, } = item;
                        const { name: metaName, title, platform, shortcut } = meta || {};
                        return {
                            name,
                            dataForm: Object.entries(dataForm || {}).reduce((map, [key, config]) => {
                                const { type, ['x-slot-scope']: xSlotScope, ['x-slot-directives']: xSlotDirectives, syncProp, inputProp, } = config;
                                const filteredDataForm = {
                                    type,
                                    'x-slot-scope': xSlotScope,
                                    'x-slot-directives': xSlotDirectives,
                                    syncProp,
                                    inputProp,
                                };
                                map[key] = filteredDataForm;
                                return map;
                            }, {}),
                            properties: properties
                                ? {
                                    ...properties,
                                    properties: Object.entries(properties.properties || {}).reduce((map, [key, config]) => {
                                        const { type, title } = config;
                                        map[key] = {
                                            type,
                                            title,
                                        };
                                        return map;
                                    }, {}),
                                }
                                : undefined,
                            emitEvents,
                            events,
                            meta: meta
                                ? {
                                    name: metaName,
                                    title,
                                    platform,
                                    shortcut,
                                }
                                : undefined,
                            compConfig,
                            inputProps,
                            syncProps,
                            isContainer,
                        };
                    }),
                };
                return map;
            }, {}), undefined, 2),
        },
        'utils/adapter/lowcodeModules.tpl.js': {
            subPackageNames: calses.filter((item) => { var _a; return (_a = item.extra) === null || _a === void 0 ? void 0 : _a.rootPath; }).map((cals) => { var _a; return (_a = cals.extra) === null || _a === void 0 ? void 0 : _a.rootPath; }),
        },
    };
    templatesData['index.jsx'] = {
        isSandbox,
        adminPortalKey: (deployOptions === null || deployOptions === void 0 ? void 0 : deployOptions.mode) === types_1.DEPLOY_MODE.UPLOAD ? appKey : `${appKey}-preview`,
        processCssUnit,
    };
    fileCodeMap = generateFilesFromTemplate({
        templatesData,
        fileCodeMap,
        baseTargetPath: appBuildDir,
    });
}
function getComponentImportStringArr(components, componentImportStringArr = []) {
    components.map(async (component) => {
        const { name, materialName, materialVersion, variableName, entries } = component;
        // const fullName = `${materialName}_${name}`
        // 这里将头字母变成大写是为了能在jsx中以<XXX/>去引用组件
        let importString = '';
        if (entries === null || entries === void 0 ? void 0 : entries.components) {
            importString = `import { ${name} as ${(0, util_1.upperFirst)(variableName)} } from '${path_1.default.posix.join(`/src/libraries/${materialName}@${materialVersion}`, entries === null || entries === void 0 ? void 0 : entries.components)}'`;
        }
        else if (entries === null || entries === void 0 ? void 0 : entries.entry) {
            const componentsLibVariableName = (0, util_1.camelcase)(`${materialName}`);
            importString = `const { ${name}: ${(0, util_1.upperFirst)(variableName)} } = ${componentsLibVariableName}.components`;
        }
        else {
            importString = `import ${(0, util_1.upperFirst)(variableName)} from '/src/libraries/${`${materialName}@${materialVersion}`}/components/${name}'`;
        }
        if (!componentImportStringArr.includes(importString)) {
            componentImportStringArr.push(importString);
        }
    });
    return componentImportStringArr.filter((item) => !!item);
}
function getEntryImportStringArr(materialInfoList = [], entryImportStringArr = []) {
    materialInfoList.forEach(({ materialName, version, entries }) => {
        if (!((entries === null || entries === void 0 ? void 0 : entries.actions) && (entries === null || entries === void 0 ? void 0 : entries.components)) && (entries === null || entries === void 0 ? void 0 : entries.entry)) {
            const componentsLibVariableName = (0, util_1.camelcase)(`${materialName}`);
            const importComponentLibString = `import ${componentsLibVariableName} from '${path_1.default.posix.join(`/src/libraries/${materialName}@${version}`, entries === null || entries === void 0 ? void 0 : entries.entry)}'`;
            if (!entryImportStringArr.includes(importComponentLibString)) {
                entryImportStringArr.push(importComponentLibString);
            }
        }
    });
    return entryImportStringArr || [];
}
function getActionImportStringArr(originActionList, actionImportStringArr = []) {
    originActionList.map((action) => {
        if (action.type === weapps_core_1.ActionType.Material) {
            pushActionToImportStringArr(action, actionImportStringArr);
        }
    });
    return actionImportStringArr;
}
function pushActionToImportStringArr(listenerInstance, actionImportStringArr) {
    const { name, materialName, materialVersion, variableName, entries } = listenerInstance;
    const componentsLibVariableName = (0, util_1.camelcase)(`${materialName}`);
    const importString = (entries === null || entries === void 0 ? void 0 : entries.actions)
        ? `import { ${name} as ${variableName} } from '${path_1.default.posix.join(`/src/libraries/${materialName}@${materialVersion}`, entries === null || entries === void 0 ? void 0 : entries.actions)}'`
        : (entries === null || entries === void 0 ? void 0 : entries.entry)
            ? `const { ${name}: ${variableName} } = ${componentsLibVariableName}.actions`
            : `import ${variableName} from '/src/libraries/${materialName}@${materialVersion}/actions/${name}'`;
    if (actionImportStringArr.includes(importString)) {
        return;
    }
    actionImportStringArr.push(importString);
}
function unshiftAllOfficialActionToImportStringArr(officialLib, actionImportStringArr) {
    let importString = '';
    if (officialLib) {
        const { entries } = officialLib;
        const componentsLibVariableName = (0, util_1.camelcase)(`${officialLib.name}`);
        importString = (entries === null || entries === void 0 ? void 0 : entries.actions)
            ? `import * as $$OFFICIAL_ACTION from '${path_1.default.posix.join(`/src/libraries/${officialLib.name}@${officialLib.version}`, entries === null || entries === void 0 ? void 0 : entries.actions)}'`
            : (entries === null || entries === void 0 ? void 0 : entries.entry)
                ? `const $$OFFICIAL_ACTION  = ${componentsLibVariableName}.actions`
                : '';
        // TODO: 最早的散装函数需要全部手动引用
        // : `import ${variableName} from '/src/libraries/${materialName}@${materialVersion}/actions/${name}'`;
    }
    else {
        importString = `const $$OFFICIAL_ACTION = {}`;
    }
    if (actionImportStringArr.includes(importString)) {
        return;
    }
    actionImportStringArr.unshift(importString);
}
function generateFilesFromTemplate({ templatesData, fileCodeMap = {}, baseTargetPath = '/src', }) {
    // eslint-disable-next-line no-restricted-syntax
    for (let file in templatesData) {
        let [templatePath, targetPath] = file.split('|').filter((item) => item === null || item === void 0 ? void 0 : item.trim());
        if (!/^\/src\//.test(templatePath)) {
            templatePath = path_1.default.posix.join('/src', templatePath);
        }
        if (!targetPath) {
            targetPath = templatePath.replace(_1.TEMPLATE_REGEX, '$1');
        }
        if (/^\/src\//.test(targetPath)) {
            targetPath = path_1.default.relative('/src', targetPath);
        }
        if (!template_1.default[templatePath]) {
            throw new Error(`not found ${templatePath}`);
        }
        const tplStr = template_1.default[templatePath].code;
        const generatedCode = (0, lodash_1.template)(tplStr, { interpolate: /<%=([\s\S]+?)%>/g })(templatesData[file]);
        const outFile = path_1.default.join(baseTargetPath, targetPath);
        fileCodeMap[outFile] = {
            code: generatedCode,
        };
    }
    return fileCodeMap;
}
exports.generateFilesFromTemplate = generateFilesFromTemplate;
